/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */

#include <config.h>

#include "tracker-status-icon.h"

#include <gtk/gtk.h>
#include <string.h>
#include <glib/gi18n-lib.h>

#include <libnotify/notify.h>

struct _TrackerStatusIconPrivate
{
  GtkMenu    *menu;
  GtkWidget  *stats_dialog;

  NotifyNotification *notification;
};

G_DEFINE_TYPE (TrackerStatusIcon, tracker_status_icon, GTK_TYPE_STATUS_ICON);

typedef struct
{
  TrackerStatusIcon *icon;
  char *summary;
  char *body;
}TrackerStatusIconNotify;

static gboolean tracker_status_icon_show_notification (gpointer user_data);

static void
tracker_status_icon_finalize (GObject *object)
{
  TrackerStatusIcon *icon = TRACKER_STATUS_ICON (object);

  if (icon->priv->notification)
    g_object_unref (icon->priv->notification);
  icon->priv->notification = NULL;

  if (icon->priv->menu)
    gtk_widget_destroy (GTK_WIDGET(icon->priv->menu));
  icon->priv->menu = NULL;

  if (icon->priv->stats_dialog)
    gtk_widget_destroy (icon->priv->stats_dialog);
  icon->priv->stats_dialog = NULL;

  G_OBJECT_CLASS (tracker_status_icon_parent_class)->finalize (object);
}

static void
tracker_status_icon_init (TrackerStatusIcon *icon)
{
  icon->priv = G_TYPE_INSTANCE_GET_PRIVATE (icon, TRACKER_TYPE_STATUS_ICON, TrackerStatusIconPrivate);
}

TrackerStatusIcon *
tracker_status_icon_new ()
{
  return g_object_new (TRACKER_TYPE_STATUS_ICON,
                       "icon-name", "tracker",
                       NULL);
}

static void
tracker_status_icon_spawn_command (TrackerStatusIcon *icon,
                                const char     *command,
                                const char     *error_format)
{
  GdkScreen *screen;
  GError    *error;

  g_return_if_fail (TRACKER_IS_STATUS_ICON (icon));

	screen = gdk_screen_get_default ();

  error = NULL;

  if (!gdk_spawn_command_line_on_screen (screen, command, &error))
    {
      GtkWidget *message_dialog;

      message_dialog = gtk_message_dialog_new (NULL,
					       GTK_DIALOG_DESTROY_WITH_PARENT,
					       GTK_MESSAGE_ERROR,
					       GTK_BUTTONS_CLOSE,
                                               error_format,
					       error->message);
      gtk_window_set_resizable (GTK_WINDOW (message_dialog), FALSE);

      g_signal_connect (message_dialog, "response",
			G_CALLBACK (gtk_widget_destroy),
			NULL);

      gtk_widget_show (message_dialog);

      g_error_free (error);
    }
}

static void
tracker_status_icon_start_indexing (TrackerStatusIcon *icon)
{
  // XXX Start/Pause indexing XXX
  return;
}

static void
tracker_status_icon_search(TrackerStatusIcon *icon)
{
  tracker_status_icon_spawn_command (icon,
                                  "tracker-search-tool",
                                  _("There was an error displaying tracker-search-tool:\n %s"));
}

static void
tracker_status_icon_preferences (TrackerStatusIcon *icon)
{
  tracker_status_icon_spawn_command (icon,
                                  "tracker-preferences",
                                  _("There was an error displaying tracker-preferences:\n %s"));
}

static void
tracker_status_icon_help (TrackerStatusIcon *icon)
{
#if 0
  tracker_status_icon_spawn_command (icon,
                                  "gnome-open ghelp:user-guide?goscustdesk-90",
                                  _("There was an error displaying trakcer-help:\n %s"));
#endif
}

static void
tracker_status_icon_about (TrackerStatusIcon *icon)
{

  g_return_if_fail (TRACKER_IS_STATUS_ICON (icon));

  const char *authors[] = {
    "somebody@domain.com",
    NULL
  };
  char *license;
  char *translators;

  license = _("Licensed under the GNU General Public License Version 2\n\n"
              "Tracker is free software; you can redistribute it and/or\n"
              "modify it under the terms of the GNU General Public License\n"
              "as published by the Free Software Foundation; either version 2\n"
              "of the License, or (at your option) any later version.\n\n"
              "Tracker is distributed in the hope that it will be useful,\n"
              "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
              "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the\n"
              "GNU General Public License for more details.\n\n"
              "You should have received a copy of the GNU General Public License\n"
              "along with this program; if not, write to the Free Software\n"
              "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA\n"
              "02110-1301, USA.\n");

  /* Translators comment: put your own name here to appear in the about dialog. */
  translators = _("translator-credits");

  if (!strcmp (translators, "translator-credits"))
    translators = NULL;

  gtk_show_about_dialog (NULL,
                         "name",               _("Meta Tracker"),
                         "comments",           _("Desktop Search Tool"),
                         "version",            VERSION,
                         "license",            license,
                         "authors",            authors,
                         "translator-credits", translators,
                         "logo-icon-name",     "tracker",
                         NULL);
}

static void
tracker_status_icon_popup_menu (GtkStatusIcon *status_icon,
			     guint          button,
			     guint32        timestamp)
{
  TrackerStatusIcon *icon = TRACKER_STATUS_ICON (status_icon);
  GtkWidget      *item;
  GSList         *l;
  TrackerStatusIconNotify *a;

  icon->priv->menu = (GtkMenu*) gtk_menu_new ();

  item = gtk_image_menu_item_new_with_mnemonic (_("_Search"));
  g_signal_connect_swapped (item, "activate",
                            G_CALLBACK (tracker_status_icon_search), icon);
  gtk_widget_show (item);
  gtk_menu_shell_append (GTK_MENU_SHELL (icon->priv->menu), item);

  // XXX Should use _Start or Pa_use based on current status XXX
  item = gtk_image_menu_item_new_with_mnemonic (_("_Start/Pa_use Indexing(NYI)"));
  g_signal_connect_swapped (item, "activate",
                            G_CALLBACK (tracker_status_icon_start_indexing), icon);
  gtk_widget_show (item);
  gtk_menu_shell_append (GTK_MENU_SHELL (icon->priv->menu), item);

  item = gtk_image_menu_item_new_with_mnemonic (_("_Preferences"));
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
                                 gtk_image_new_from_stock (GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU));
  g_signal_connect_swapped (item, "activate",
                            G_CALLBACK (tracker_status_icon_preferences), icon);
  gtk_widget_show (item);
  gtk_menu_shell_append (GTK_MENU_SHELL (icon->priv->menu), item);

  item = gtk_separator_menu_item_new ();
  gtk_widget_show (item);
  gtk_menu_shell_append (GTK_MENU_SHELL (icon->priv->menu), item);

  item = gtk_image_menu_item_new_with_mnemonic (_("_Help(NYI)"));
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
                                 gtk_image_new_from_stock (GTK_STOCK_HELP, GTK_ICON_SIZE_MENU));
  g_signal_connect_swapped (item, "activate",
                            G_CALLBACK (tracker_status_icon_help), icon);
  gtk_widget_show (item);
  gtk_menu_shell_append (GTK_MENU_SHELL (icon->priv->menu), item);

  item = gtk_image_menu_item_new_with_mnemonic (_("_About"));
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
                                 gtk_image_new_from_stock (GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU));
  g_signal_connect_swapped (item, "activate",
                            G_CALLBACK (tracker_status_icon_about), icon);
  gtk_widget_show (item);
  gtk_menu_shell_append (GTK_MENU_SHELL (icon->priv->menu), item);

  gtk_menu_popup (GTK_MENU (icon->priv->menu), NULL, NULL,
		  gtk_status_icon_position_menu, icon,
		  button, timestamp);
  if (button == 0)
    gtk_menu_shell_select_first (GTK_MENU_SHELL (icon->priv->menu), FALSE);
}

static void
tracker_status_icon_activate (GtkStatusIcon *icon)
{
  tracker_status_icon_preferences (TRACKER_STATUS_ICON (icon));
}

static void
tracker_status_icon_update_tooltip (TrackerStatusIcon *icon)
{
  char *tooltip;
  // XXX get trackerd status XXX
  // tooltip = g_strdup_printf (_("Tracker: %s"), "Initial Indexing");
  
  gtk_status_icon_set_tooltip (GTK_STATUS_ICON (icon), tooltip);
  g_free (tooltip);
}

static void
tracker_status_icon_class_init (TrackerStatusIconClass *klass)
{
  GObjectClass       *gobject_class     = G_OBJECT_CLASS (klass);
  GtkStatusIconClass *status_icon_class = GTK_STATUS_ICON_CLASS (klass);

  gobject_class->finalize     = tracker_status_icon_finalize;
  status_icon_class->activate   = tracker_status_icon_activate;
  status_icon_class->popup_menu = tracker_status_icon_popup_menu;

  g_type_class_add_private (gobject_class, sizeof (TrackerStatusIconPrivate));
}

static void
tracker_status_handle_notification_closed (TrackerStatusIcon *icon)
{
  g_object_unref (icon->priv->notification);
  icon->priv->notification = NULL;
}

static gboolean
tracker_status_icon_show_notification (gpointer user_data)
{
#define NOTIFICATION_TIMEOUT 5

  GError     *error;
  TrackerStatusIconNotify *a = (TrackerStatusIconNotify *)user_data;
  TrackerStatusIcon *icon    = a->icon;
  char *summary = a->summary;
  char *body = a->body;

  if (!notify_is_initted () &&  !notify_init (_("Meta Tracker")))
  {
    g_printerr (_("Error initializing libnotify\n"));
    g_free (body);
    g_free (summary);
    g_free (user_data);
    return FALSE;
  }

  if (icon->priv->notification)
  {
    notify_notification_close (icon->priv->notification, NULL);
    g_object_unref (icon->priv->notification);
    icon->priv->notification = NULL;
  }

  icon->priv->notification = notify_notification_new_with_status_icon (summary,
                                              body,
                                              "tracker",
                                              GTK_STATUS_ICON (icon));
  g_free (body);
  g_free (summary);
  g_free (user_data);

  g_signal_connect_swapped (icon->priv->notification, "closed",
                            G_CALLBACK (tracker_status_handle_notification_closed),
                            icon);

  notify_notification_set_timeout (icon->priv->notification,
                                   NOTIFICATION_TIMEOUT * 1000);

  error = NULL;
  if (!notify_notification_show (icon->priv->notification, &error))
    {
      g_printerr (_("Error while displaying notification bubble: %s\n"),
                  error->message);
      g_error_free (error);
    }

  g_free (body);
  g_free (summary);
  g_free (user_data);

#undef NOTIFICATION_TIMEOUT
  return FALSE;
}

gint
main (gint argc,
      gchar * argv[])
{
  gtk_init(&argc, &argv);

  TrackerStatusIcon *status_icon = tracker_status_icon_new();

  gtk_main ();
	return 0;
}
